# простая торговая панель
set hasMessage 0
set autoTrade 0
oo::class create TradePanel {
variable W			; # имя окна Tk
variable Symbol		; # символ
variable Bid		; # значение Bid
variable Ask		; # значение Ask
variable LabelBid	; # имя виджета с Bid 
variable LabelAsk	; # имя виджета с Ask
variable Outgoing 	; # простая очередь исходящих (чтобы MT её считывал, сами мы MT вызывать не можем)
variable Orders		; # ордера в рынке
variable Tree
constructor { w symbol } {
	set W $w
	set Symbol $symbol
	set Bid 0
	set Ask 0
	set LabelBid ""
	set LabelAsk ""
	set Outgoing {}
	array set Orders {}
	my Draw
	my ShowOrderList
}
destructor {
	catch { destroy $W }
}
method Print { args } {
	lappend Outgoing [ list 0 $args ]
	global hasMessage
	incr hasMessage
}
method Draw { } {
	# создаём окно
	set W [ toplevel $W ]	; # toplevel окно
	wm protocol $W WM_DELETE_WINDOW { bell }	; # запретим закрывать по "крестику" 
	wm title $W $Symbol		; # выведем заголовок
	# сделаем выпадающее меню 
	set mainMenu [ menu $W.menu -tearoff 0 ]
	$mainMenu add cascade -label "Market" -menu $W.menu.market
	set marketMenu [ menu $W.menu.market -tearoff 0 ]
	$marketMenu add command -label "Orders" -command [ list [ self ] ShowOrderList ]
	$mainMenu add cascade -label "Setup" -menu $W.menu.setup
	set setupMenu [ menu $W.menu.setup -tearoff 0 ]
	$setupMenu add checkbutton -label "Autotrade" -variable ::autoTrade 
	$mainMenu add cascade -label "Help" -menu $W.menu.help
	set helpMenu [ menu $W.menu.help -tearoff 0 ]
	$helpMenu add command -label "About" -command [ list tk_messageBox -parent $W -title "TradePanel" -message "ATcl demonstation" ]
	# назначим его как меню окна
	$W configure -menu $W.menu
	# и в окне всякие элементы
	set f [ frame $W.panel ]
	label $f.title -text $Symbol -fg grey -font "{Arial Black}"
	set LabelBid [ label $f.bid  -fg grey -font "{Arial Black}" ]
	set LabelAsk [ label $f.ask  -fg grey -font "{Arial Black}" ]
	button $f.buy  -text "Buy"  -fg blue -font "{Arial Black}" -command [ list [ self ] OnBuyPressed ] ; # кнопка Buy - при нажатии вызовет собственный метод OnBuyPressed
	button $f.sell -text "Sell" -fg red -font "{Arial Black}" -command [ list [ self ] OnSellPressed ]
	# проще всего располагать элементы в таблице
	grid $f.title -row 0 -column 0 -columnspan 2 -sticky "ew"
	grid $f.bid -row 1 -column 0 -sticky "e"
	grid $f.ask -row 1 -column 1 -sticky "w"
	grid $f.sell -row 2 -column 0 -sticky "nsew"
	grid $f.buy -row 2 -column 1 -sticky "nsew"
	grid columnconfigure $f 0 -uniform same
	grid columnconfigure $f 1 -uniform same
	# фрейм с контролами - на всё окно
	pack $f -fill both -expand yes
	
	return $W
}
method OnBuyPressed {} {
	# при нажатии кнопки Buy :
	# добавить в исходящий список сообщение 
	global hasMessage
	incr hasMessage
	lappend Outgoing [ list "buy" [ clock milliseconds ] ]
}
method OnSellPressed {} {
	# при нажатии кнопки Sell :
	# добавить в исходящий список сообщение 
	global hasMessage
	incr hasMessage
	lappend Outgoing [ list "sell" [ clock milliseconds ] ]
}

# метод будет вызываться из MT когда приходит новый тик
# поменяем значения в виджетах и расчетим заодно 
method OnTick { bid ask } {
	if { $bid == $Bid } {
		$LabelBid configure -fg grey
	} elseif { $bid > $Bid } {
		$LabelBid configure -fg blue
	} else {
		$LabelBid configure -fg red
	}
	if { $ask == $Ask } {
		$LabelAsk configure -fg grey
	} elseif { $ask > $Ask } {
		$LabelAsk configure -fg blue
	} else {
		$LabelAsk configure -fg red
	}
	set Bid $bid
	set Ask $ask
	$LabelAsk configure -text $Ask
	$LabelBid configure -text $Bid
}
# метод будет вызываться из MT чтобы считать все сообщения
method Messages {} {
	set msgs $Outgoing
	set Outgoing {}
	return $msgs
}
# метод вызывается периодически с полным списом ордеров
# надо обновить внутренние структуры
# и вернуть триплет (список из трёх списков)
#   1 - список новых ордеров
#   2 - ордера которые поменялись
#	3 - ордера которые удалены
method OnOrderList { orderList } {
	set created {}
	set changed {}
	set removed {}
	foreach order $orderList {
		set ticket [ lindex $order 0 ]
		set known($ticket) $order
		if { ! [ info exists Orders($ticket) ] } {
			# новый тикет 
			# добавить в список свежесозданных
			lappend created $ticket
			set Orders($ticket) $order
			continue
		}
		if { $Orders($ticket) != $order } {
			# что-то поменялось 
			# добавить в список изменённых
			set Orders($ticket) $order
			lappend changed $ticket
		}
	}
	foreach ticket [ array names Orders ] {
		if { ! [ info exists known($ticket) ] } {
			# прежнего ордера нет в новом списке - значит удалён
			lappend removed $ticket
			unset Orders($ticket)
		}
	}
	# обновить грфику :-)
	foreach ticket $removed {
		my OnRemoveOrder $ticket
	}
	foreach ticket $changed {
		my OnChangeOrder $ticket
	}
	foreach ticket $created {
		my OnCreateOrder $ticket
	}
	return [ list $created $changed $removed ]
}
method ShowOrderList {} {
	set w $W.orderListWindow
	if [ winfo exists $w ] {
		wm deiconify $w
		raise $w
		focus $w
		return
	} else {
		toplevel $w
		wm title $w "OrderList"
		wm protocol $w WM_DELETE_WINDOW [ list wm withdraw $w ]
		set f [ frame $w.area ]
		set tree [ ttk::treeview $f.tree -yscrollcommand [ list $f.vscroll set ] -xscrollcommand [ list $f.hscroll set ] \
			-columns {openTime ticket type lots openPrice stopLoss takeProfit swap commision comment} -show headings ]
		$tree heading openTime -text "Time"
		$tree heading ticket -text "Ticket"
		$tree heading type -text "Type"
		$tree heading lots -text "Lots"
		$tree heading openPrice -text "Price"
		$tree heading stopLoss -text "Stop loss"
		$tree heading takeProfit -text "Take profit"
		$tree heading swap -text "Swap"
		$tree heading commision -text "Comission"
		$tree heading comment -text "Comment"
		
		$tree column openTime -width 100 -stretch false
		$tree column ticket -width 100 -stretch false
		$tree column type -width 100 -stretch false
		$tree column lots -width 100 -stretch false
		$tree column openPrice -width 100 -stretch false
		$tree column stopLoss -width 100 -stretch false
		$tree column takeProfit -width 100 -stretch false
		$tree column swap -width 100 -stretch false
		$tree column commision -width 100 -stretch false
		$tree column comment -width 100 -stretch true
		
		scrollbar $f.vscroll -orient vert -command [ list $f.tree yview ]
		scrollbar $f.hscroll -orient hor -command [ list $f.tree xview ]
		grid $f.tree -row 0 -column 0 -sticky "nsew"
		grid $f.vscroll -row 0 -column 1 -sticky "ns"
		grid $f.hscroll -row 1 -column 0 -sticky "ew"
		grid columnconfigure $f 0 -weight 1 
		grid rowconfigure $f 0 -weight 1 
		pack $f -fill both -expand yes
		set Tree $tree
	}
}
method OnCreateOrder { ticket } {
	my Print "OnCreateOrder $Orders($ticket)"
	if { [ catch {
	set item [ $Tree insert {} end -id $ticket ]
	lassign $Orders($ticket) ticket type time symbol lots price stopLoss takeProfit comment swap commision 
	my Print "ticket=$ticket item=$item symbol=$symbol"
	switch $type {
		0 {
			set type "Buy"
		}
		1 { 
			set type "Sell"
		}
		2 { 
			set type "BuyLimit"
		}
		3 { 
			set type "BuyStop"
		}
		4 { 
			set type "SellLimit"
		}
		5 { 
			set type "SellStop"
		}
	}
	$Tree set $item openTime $time 
	$Tree set $item type $type
	$Tree set $item ticket $ticket
	$Tree set $item lots $lots
	$Tree set $item openPrice $price
	$Tree set $item stopLoss $stopLoss
	$Tree set $item takeProfit $takeProfit
	$Tree set $item swap $swap
	$Tree set $item commision $commision
	$Tree set $item comment $comment
	
	} msg ] } {
		my Print [ list error $msg ]
	}
}
method OnRemoveOrder { ticket } {
	$Tree delete $ticket
}
method OnChangeOrder { ticket } {
	my Print OnChangeOrder $ticket
	set item $ticket
	lassign $Orders($ticket) ticket type time symbol lots price stopLoss takeProfit comment swap commision 
	switch $type {
		0 {
			set type "Buy"
		}
		1 { 
			set type "Sell"
		}
		2 { 
			set type "BuyLimit"
		}
		3 { 
			set type "BuyStop"
		}
		4 { 
			set type "SellLimit"
		}
		5 { 
			set type "SellStop"
		}
	}
	$Tree set $item openTime $time
	$Tree set $item type $type
	$Tree set $item lots $lots
	$Tree set $item openPrice $price
	$Tree set $item stopLoss $stopLoss
	$Tree set $item takeProfit $takeProfit
	$Tree set $item swap $swap
	$Tree set $item commision $commision
	$Tree set $item comment $comment
}

export OnBuyPressed OnSellPressed OnOrderList
export OnTick Messages ShowOrderList
} ; # /class TradePanel

##### TEST CASE для автономного запуска #####
if { [ info exists argv0 ] && $argv0 == [ info script ] } {
	package require Tk
	set panel [ TradePanel new .tradePanel "EURUSD" ]
	$panel OnTick 1.2345 1.1234
	update
	wm withdraw .
	tkwait window .tradePanel
	exit
}
