package require struct::queue 	; # нужны очереди
package require websocket		; # вебсокет
package require http			; # протокол http
package require tls				; # протокол tls
package require json			; # данные идут в json
http::register https 443 [list ::tls::socket ]	; # https (и wss) - защита tls
proc Log { args } {
	if { [ catch {
		set f [ open "WebSockBinance.log" "a" ]
		if { $f != {} } {
			puts -nonewline $f "[ clock format [ clock seconds ] ] "
			puts $f [ join $args " " ]
			flush $f
			close $f
		}
	} msg ] } {
		catch { puts stderr "Log error: $msg" }
	}
}
oo::class create WebSockBinance {
	variable Symbol 
	variable WS
	variable Out
	variable Connected
	constructor { symbolName } {
		set WS {}
		set Symbol 	$symbolName
		set Out		[ ::struct::queue new ]
		set Connected false
		Log "constructor ok Symbol=$Symbol"
	}
	destructor {
		catch {
			if { $WS != {} } {
				catch { ::websocket::close $WS }	; # если сокетоткрыт - то закрыть игнориуюя ошибки
			}
			if { $Out != {} } {
				catch { $Out destroy }	; # удалить объект - очередь
			}
		}
		Log "destructor ok"
	}
	method Start {} {
		Log "method Start"
		if { [ catch { 
			# открыть веб-сокет на тикеры
			set url "wss://stream.binance.com:9443/ws/[string tolower ${Symbol}]@ticker"
			# отвечать будет кол-бек класса - Handler
			set WS [ websocket::open $url [ list [ self ] Handler ] ]
		} msg ] } {
			# ошибка при открытии сокета 
			Log "Start failed"
			return false
		}
		Log "Start ok url=$url"
		#update
		# сокет открыт
		return true
	}
	method StartAndConnect {} {
		if { [ my Start ] } {
			while { ! [ my Connected ] } {
				update
			}
		}
	}
	method Stop {} {
		Log "method Stop"
		if { $WS != {} } {
			# просто закрыть сокет
			::websocket::close $WS
			set WS {}
		}
		Log "Stop done"
	}
	method Ready {} {
		if { $WS != {} } {
			return true ;	# сокет открыт - мы по идее готовы
		}
		return false	; 	# нет признаков готовности - не готовы
	}
	method Handler { sock type text } {
		switch -glob -nocase -- $type {
			co* {
				# "Co-nnect" и прочие прозводные - прошло соединение
				Log "Connect"
				set Connected true
			}
			te* {
				Log "Text $text"
				# "Te-xt" и подобные - получены данные
				if { [ catch {
					# данные приходят в json
					set data [ ::json::json2dict $text ]
				} msg ] } {
					# не удалось разобрать json
					return
				}
				if { ! [ dict exists $data "E" ] || ! [ dict exists $data "b" ] || ! [ dict exists $data "a" ] } {
					# нет необходимых полей
					return
				} 
				set time [ dict get $data "E" ]
				set sec [ expr { $time / 1000 } ]
				set msc [ expr { $time - $sec * 1000 } ]
				if { $msc >= 1000 } {
					# leap
					set msc 1000
				}
				set bid [ dict get $data "b" ]
				set ask [ dict get $data "a" ]
				if { $Out != {} } {
					$Out put [ list $sec $msc $bid $ask ] 	; 	# отдадим в очередь
					while { [ $Out size ] > 1024 } {
						# при переполнении - удаляем старые
						$Out get
					}
				}
			}
			er* {
				Log "Error $text"
				# "Er-ror - какие-то ошибки
			}
			clo* -
			dis* {
				set Connected false
				Log "Disconnect"
				# "Clo-se или Dis-connect - разрыв
				catch { my Stop }
				# через 15 сек - попробуем переподключится
				after 15000 [ list [ self ] Start ]
			}
			default {
				# ходят ещё разные служебные пакеты, но их можно не печатать
			}
		}
	}
	method Get {} {
		if { [ $Out size ] == 0 } {
			# если не задана исходящая очередь или она пуста, можно генерить ошибку
			return {}
		}
		tailcall $Out get	; # делегировать в Get очереди
	}
	method Connected {} {
		return $Connected
	}
	export Connected
	export Start StartAndConnect Stop Ready Get	; # публичные методы
	export Handler				; # свой колбек тоже надо экспортировать
}
########### TESTS ############
if { [ info exists argv0 ] && $argv0 == [ info script ] } {
	set ethbtc [ WebSockBinance new "ethbtc" ]
	set btcusdt [ WebSockBinance new "btcusdt" ]
	$ethbtc Start
	while { ! [ $ethbtc Connected ] } {
		after 100
		update
	}
	puts "EthBtc connected"
	$btcusdt Start
	while { [ $ethbtc Ready ] && [ $btcusdt Ready ] } {
		update
		while { [ set record [ $ethbtc Get ] ] != {} } {
			puts "ethbtc $record"
		}
		while { [ set record [ $btcusdt Get ] ] != {} } {
			puts "btcusdt $record"
		}
		after 100
	}
	$ethbtc Stop
	$ethbtc destroy
}
